<?php
require_once ('Matrix.php'); // matrix inversion
class PolynomialApproximation
{
//    function __construct() {
//    }
    public $debugLevel = 1;
    public $debugFile = null;
    
    public function polynomialApproximation1d($data, $N){
        //$my_array = array_fill(0, $size_of_the_array, $some_data);
        if ($this->debugFile === null){
            $this->debugLevel = 1;
        }
        $S=array_fill(0,2*$N +1,0);  // new double [2*N+1];
        $SF=array_fill(0,$N+1,0); // new double [N+1];
        for ($i=0; $i < sizeof($data); $i++){
            $wxn=(sizeof($data[$i]) > 2)? $data[$i][2] : 1.0;
            if ($wxn > 0.0){ // save time on 0.0 that can be used to mask out some samples
                $f=$data[$i][1];
                $x=$data[$i][0];
                for ($j=0; $j <= $N; $j++){
                    $S[$j]+=$wxn;
                    $SF[$j]+=$wxn * $f;
                    $wxn *= $x;
                }
                for ($j = $N+1; $j < 2*$N;$j++){
                    $S[$j] += $wxn;
                    $wxn *= $x;
                }
                $S[2*$N] += $wxn;
                if ($this->debugLevel > 1) {
                    if (sizeof($data[$i]) > 2)
                        fprintf($this->debugFile, "polynomialApproximation1d() |" . $i . "|: x=|" . $data[$i][0] . "| f(x)=|" . $data[$i][1] . "| (w=\t|" . $data[$i][2] . "|\n)");
                    else
                        fprintf($this->debugFile, "polynomialApproximation1d() |" . $i, "|: x=|" . $data[$i][0] . "| f(x)=|" . $data[$i][1] . "|\n)");
                }
            }
        }
        $aM = Matrix::zeroMatrix($N+1,$N+1);
        $aB = Matrix::zeroMatrix($N+1,1);
        for ($i=0; $i <= $N; $i++) {
            $aB[$i][0] = $SF[$i];
            for ($j = 0; $j <= $N; $j++) $aM[$i][$j]= $S[$i + $j];
        }
        $N1 = $N;
        $M =  new Matrix($aM);
        $B =  new Matrix($aB);
        
        if ($this->debugLevel > 1){
            fprintf($this->debugFile,
                "\npolynomialApproximation1d(data,".$N.") M:\n");
            $M->print($this->debugFile);
            fprintf($this->debugFile,
                "\npolynomialApproximation1d() B:\n");
            $B->print($this->debugFile);
        }
            // while (!(new LUDecomposition(M)).isNonsingular() && (N1>0)){
//        $df = ($this->debugLevel > 2) ? $this->debugFile : null;
        while ($N1 >= 0) { // make N=0 legal ?
            try {
                $R = $M->solve($B); 
                
                if ($this->debugLevel > 1){
                    fprintf($this->debugFile,
                        "polynomialApproximation1d() solution=\n");
                    $R->print($this->debugFile);
                }
                $result = $R->getColumnPackedCopy();
                return $result;
                
            } catch (Exception $e) {
                echo 'Caught exception: ',  $e->getMessage(), ", reducing polynomial order\n";
                if ($N1 < 0 ) {
                    return null;
                }
                $N1--;
                $aM1 = Matrix::zeroMatrix($N+1,$N+1);
                $aB1 = Matrix::zeroMatrix($N+1,1);
                for ($i = 0; $i <= $N1; $i++) {
                    $aB1[$i][0] = $aB[$i][0];
                    for ($j = 0; $j <= $N1; $j++){
                        $aM1[$i][$j] = $aM[$i][$j];
                    }
                }
                $M =  new Matrix($aM1);
                $B =  new Matrix($aB1);
            }
        }
        return null;
    }
    
    
    
}
?>
